// main.cpp
//
// CompSci 141 / CSE 141 / Informatics 101 Spring 2013
// In-class example: a Stack ADT in C
//
// This file contains the main() function, in which we'll create and use a
// couple of stack "objects," to demonstrate the ADT concept as manifested
// in C.  (I've actually used a few non-object-oriented C++ features to keep
// the code clean, which is why I've used the .cpp extension on the two source
// files.)


// We'll need to include declarations of two things: the standard C library
// input/output functions (stdio.h) and our stack ADT (stack.h).

#include <stdio.h>
#include "stack.h"



int main()
{
	// Let's create a couple of stacks...
	stack* s1 = stack_create();
	stack* s2 = stack_create();

	// We'll start by printing out the size of the two stacks initially.
	// The printf() function is how you print output in C.  You give it
	// a series of parameters.  The first parameter is a "format string,"
	// which tells it how to use the subsequent ones.  Occurrences of the
	// % character denote chunks that should be replaced with the rest of
	// the parameters.  In the example below, the %d will be replaced with
	// the second parameter (the size of the stack).  %d means to interpret
	// that parameter as an integer.  \n is a code for a newline character.

	printf("The size of stack 1 is %d\n", stack_size(s1));
	printf("The size of stack 2 is %d\n", stack_size(s2));

	
	// Now we'll try pushing and popping from the two stacks...

	stack_push(s1, 10);
	stack_push(s2, 8);
	stack_push(s1, 20);
	stack_push(s2, stack_top(s1));
	stack_pop(s1);

	printf("The size of stack 1 is %d\n", stack_size(s1));
	printf("The top of stack 1 is %d\n", stack_top(s1));
	printf("The size of stack 2 is %d\n", stack_size(s2));
	printf("The top of stack 2 is %d\n", stack_top(s2));

	// Now that we're finished with the stacks, we should destroy them
	// (which will destroy all of the contents of both stacks).
	stack_destroy(s1);
	stack_destroy(s2);

	// Remember that main() returns an int in C and C++.  The int is typically
	// used to report an "error code" back to the operating system that
	// indicates whether the program ran successfully or not.  The usual
	// convention is that an error code of 0 means success, while a non-zero
	// error code means failure.  For many programs, there is no notion of
	// out-and-out failure, so an error code of 0 is always returned.
	return 0;
}
