// stack.h
//
// CompSci 141 / CSE 141 / Informatics 101 Spring 2013
// In-class example: a Stack ADT in C
//
// This is the header file for the Stack ADT, which declares the "public
// interface" for the ADT.  We don't want to include anything in this file
// that we don't want client code -- parts of the program that use this ADT
// -- to depend upon.



// These are preprocessor directives that, in tandem with the #endif directive
// at the end of this file, ensure that this file will only be included once
// in any source file.  The entire contents of the file (other than some of
// these comments) are surrounded in a directive that says "If the symbol
// STACK_H hasn't been defined already, the following code should remain
// in place; if it has, the code should be removed by the preprocessor."  The
// first thing that the code does is define the STACK_H symobl.  So, the
// first time you include this file in a source file, you'll get its complete
// contents.  The second time, you'll get nothing.  This means there are no
// duplicate declarations and the compiler remains happy.
#ifndef STACK_H
#define STACK_H



// We'll start with a "forward declaration" of the stack structure.  This
// allows code in other source files, such as main.cpp, to have pointers to
// stack structures, but does not allow that code to do anything that
// requires the compiler to be aware of what the structure looks like (e.g.
// allocate one using "new", access one of its fields, declare a local
// variable that is a stack structure).
//
// In this case, that's a sensible restriction, since we don't want code
// outside of stack.cpp to be aware of the implementation details of a stack.
// Restricting the rest of the program from using these details is a good
// thing, since it insulates the rest of the program against change.
struct stack;


// Most of these are fairly self-explanatory, so I won't bother to comment
// them.  In a "real" ADT, it would be imperative to thoroughly explain
// the "public behavior" of each of these functions (i.e. what they do,
// but not how they do it).
//
// I will briefly say something about stack_create() and stack_destroy().
// In C, as in C++, it is the programmer's responsibility to deallocate
// objects that have been heap-allocated.  For this reason, it is necessary
// not only to have a stack_create() function, which creates and inititalizes
// (presumably on the heap) a new stack, but also a stack_destroy() function,
// which deallocates it when it's no longer needed.
//
// It should not surprise you that the rest of the functions take a stack
// as a parameter, since these are not methods that are being called on an
// object (i.e. s->push()).  In an object-oriented language, the stack itself
// would be an implicit parameter; in C, it needs to be passed explicitly.

stack* stack_create();
void stack_destroy(stack* s);
void stack_push(stack* s, int i);
void stack_pop(stack* s);
int stack_top(stack* s);
int stack_size(stack* s);




#endif // STACK_H
